#####################################################################################
# MoVib - Moray/POV > Virtualight Interface - Vib internal editor.
# 2002 Gigi Giorgi
# gigi.giorgi@ciaoweb.it
#####################################################################################

#####################################################################################
# Prima del delivery:
# Cancellare, nella versione finale, il file MoVib_setup.tcl. Verra' creato dall'utente
# 	la prima volta che lancera' MoVib.
# Commentare al fondo di questo file l'area TEMPORANEE.
# Commentare la linea che segue:
# bind . <Control-x> {destroy .; exec wish MoVib.tcl &}
#####################################################################################

#####################################################################################
# Verifica l'esistenza di nconvert
#####################################################################################
if {[auto_execok nconvert.exe]==""} {
	wm withdraw .
	tk_messageBox -message "Nconvert not found (nconvert.exe must be in your PATH).\n\
		\nPlease install nconvert.exe first (http://www.xnview.com) and make sure that\
		nconvert.exe is in your PATH environment variable."
	exit
}

#####################################################################################
# Verifica installazione di VL nel sistema:
#####################################################################################
if {[auto_execok vlight.exe]==""} {
	wm withdraw .
	tk_messageBox -message "VirtuaLight not found (vlight.exe must be in your PATH).\n\
		\nPlease install VL first and make sure that\
		vlight.exe is in your PATH environment variable."
	exit
}
if [catch {set env(VLIGHT_ROOT)} vlrootErr] {
	wm withdraw .
	tk_messageBox -message "VirtuaLight not correctly installed.\nVLIGHT_ROOT\
		variable not set.\n\nPlease install VirtuaLight first and make sure that\
		VLIGHT_ROOT environment variable is set."
	exit
} else {
	# 
	# Non e' possibile mischiare \ con / in un path. Siccome ho gia' scritto
	# un bel po' di nomi file con / devo convertire i path che il sistema mi da'
	# in formato \ nei settaggi $env(*):
	# 
	set vlrDir \
		[file join [file dirname $env(VLIGHT_ROOT)]/[file tail $env(VLIGHT_ROOT)]]
	set tmpVlrDir $vlrDir
}

#####################################################################################
# Verifica esistenza di un TEMP valida:
#####################################################################################
if [catch {set env(TEMP)}] {
	if [file exists C:/TEMP] {
		set tmpDir C:/TEMP
	} else {
		wm withdraw .
		tk_messageBox -message "Cannot set temporary directory.\
		\n\nPlease set TEMP environment variable first."
		exit
	}
} else {
	set tmpDir [file join [file dirname $env(TEMP)]/[file tail $env(TEMP)]]
	set tmpTmpDir $tmpDir
}

#####################################################################################
# Sorgenti:
#####################################################################################
package require Img
source src/init.tcl
source src/defaults.tcl
source src/settings.tcl
source src/canvas.tcl
source src/camera.tcl
source src/files.tcl
source src/include.tcl
source src/lightUtils.tcl
source src/material.tcl
source src/options.tcl
source src/parseMeshes.tcl
source src/parseObjs.tcl
source src/parsePrims.tcl
source src/render.tcl
source src/settings.tcl
source src/treeUtils.tcl
source src/utils.tcl
source src/widgets.tcl
source src/tree.tcl
source src/gui.tcl

######################################################################################
# Settaggi finali e reset/inizializzazione scena:
######################################################################################
source MoVib_config.tcl ; # settaggi vari (File > Settings)
_ghostScale
_ghostLbox
_clearScene

######################################################################################
# Verifica packing:
######################################################################################
_scrollbars
_vibFrame
_messFrame
_renderFrame

######################################################################################
# Posizionamento MoVib win:
######################################################################################
# wm geometry . +0+0
_placeAtCenterOfScreen .

wm protocol . WM_DELETE_WINDOW _exit
wm minsize . [winfo width .] [winfo height .]
_state $oText disabled
_state $mess disabled

######################################################################################
# Se il file MoVib_setup.tcl non esiste, e' la prima volta che l'utente lancia MoVib.
# Occorre quindi fare il setup delle directories fondamentali:
######################################################################################
if ![file exists MoVib_setup.tcl] {
	tk_messageBox -message "This is the first time for MoVib on this machine.\n\
	The following MoVib directories may need to be customized for your site.\n\
	Clik OK to continue ..."
	_setupWin 1
} else {
	source MoVib_setup.tcl
}

#####################################################################################
# Verifica esistenza/creazione di una dir di output:
#####################################################################################
set outputFile(dir) $vlrDir/MoVib
if ![file exists $outputFile(dir)] {
	if [catch {file mkdir "$outputFile(dir)"} mkdirErr] {
		tk_messageBox -message "Cannot create MoVib output directory\n\
			($outputFile(dir)):\n$mkdirErr"
		exit
	}
	tk_messageBox -message \
		"Created MoVib default output directories:\n\
		$outputFile(dir)"
}

###################################################################################
# TEMPORANEE: (commentare nella versione finale)
# 
# _loadMaterialsFromArchive $archiveFile(dir)/defshaders.vs
# _loadVLfiles
# set inputFile(dir) "$inputFile(dir)/35b1"
# set inputFile(name) "prism.pov"
# _parseInputFile
# _cameraSettingsWin
# $oText see 1.0
# 
###################################################################################

