proc _canvasMenu {} {
	global scCanv canvas
	set canvas(menu) [menu $scCanv.mn -tearoff 0]
	$canvas(menu) add cascade -label "Center view to" -menu $canvas(menu).m1
	menu $canvas(menu).m1 -tearoff 0
	$canvas(menu).m1 add command -label "Camera position" \
		-command {_canvSnapViewTo cameraPosition}
	$canvas(menu).m1 add command -label "Camera look_at" \
		-command {_canvSnapViewTo cameraLookAt}
	$canvas(menu) add separator
	$canvas(menu) add command -label "Zoom in" \
		-accel "<Arrow Up>" -comm {_canvZoom All 1.1}
	$canvas(menu) add command -label "Zoom out" \
		-accel "<Arrow Down>" -comm {_canvZoom All 0.9}
}

proc _canvasBindings {} {
	global scCanv cam scCanvMn canvas
	bind .cm <Up>   {_canvZoom All 1.1}
	bind .cm <Down> {_canvZoom All 0.9}
	bind $scCanv <ButtonRelease-3> {tk_popup $canvas(menu) %X %Y}
	bind $scCanv <Motion> {_canvShowCoords %x %y}
	bind $scCanv <ButtonRelease-1> {set canvas(info2) ""}
	bind $scCanv <Leave> {
		global canvas
		set canvas(info1) ""
		set canvas(info2) ""
	}
	$scCanv bind camPos <1> {
		_applyCameraCoords info1
		_canvMark %x %y
	}
	$scCanv bind camPos <B1-Motion> {_canvDrag %x %y 1}
	$scCanv bind camLine <1> {
		_applyCameraCoords info2
		_canvMark %x %y
	}
	$scCanv bind camLine <B1-Motion> {_canvDrag %x %y 2}
	$scCanv bind camPos <Enter> {$scCanv itemconf camPos -fill cyan -outline cyan}
	$scCanv bind camPos <Leave> {$scCanv itemconf camPos -fill yellow -outline yellow}
	$scCanv bind camLine <Enter> {$scCanv itemconf camLine -fill cyan}
	$scCanv bind camLine <Leave> {$scCanv itemconf camLine -fill yellow}
}

proc _canvShowCoords {x y} {
	# 
	# Visualizza in label le coordinate attuali del cursore mouse
	# 
	global canvas scCanv
	set cx [format %.3f [lindex [$scCanv coords center] 0]]
	set px [format %.3f [$scCanv canvasx $x]]
	set dx [format %.3f [expr ($cx - $px)/-$canvas(zoom)]]
	set cy [format %.3f [lindex [$scCanv coords center] 1]]
	set py [format %.3f [$scCanv canvasy $y]]
	set dy [format %.3f [expr ($cy - $py)/$canvas(zoom)]]
	set canvas(info1) "$dx $dy"
}

proc _drawGrid {} {
	global scCanv canvas
	$scCanv create line 0 0 0 0 -tags "All grid center"
	for {set n 1} {$n<=$canvas(gridDim)} {incr n} {
		$scCanv create line  $n $canvas(gridDim)  $n -$canvas(gridDim) \
			-fill grey50 -tags "All grid"
		$scCanv create line -$n $canvas(gridDim) -$n -$canvas(gridDim) \
			-fill grey50 -tags "All grid"
		$scCanv create line $canvas(gridDim)  $n -$canvas(gridDim)  $n \
			-fill grey50 -tags "All grid"
		$scCanv create line $canvas(gridDim) -$n -$canvas(gridDim) -$n \
			-fill grey50 -tags "All grid"
	}
	$scCanv create line 0 $canvas(gridDim) 0 -$canvas(gridDim) \
		-fill grey70 -tags "All grid"
	$scCanv create line $canvas(gridDim) 0 -$canvas(gridDim) 0 \
		-fill grey70 -tags "All grid"
	$scCanv scale grid 0 0 $canvas(zoom) $canvas(zoom)
	$scCanv config -scrollregion [$scCanv bbox grid]
}

proc _drawCameraView {type v1 v2} {
	global cam scCanv canvas
	set canvas(2dView) $type
	$scCanv delete cam
	# 
	# I valori z devono essere invertiti [*-1] perche' il canvas Tk ha l'asse
	# Y positivo rivolto verso il basso
	# 
	set ncamP$v2 [expr ($cam(tmpP$v2)*-1)]
	set ncamL$v2 [expr ($cam(tmpL$v2)*-1)]
	set ncamL ncamL$v2
	set ncamP ncamP$v2
	$scCanv create text $cam(tmpL$v1) [set $ncamL] \
		-text "Camera look_at" -fill yellow -anchor nw -tags "All cam cllabel"
	$scCanv create line $cam(tmpP$v1) [set $ncamP] $cam(tmpL$v1) [set $ncamL] \
		-fill yellow -arrow last -arrowshape {20 20 6} -tags "All cam camLine"
	$scCanv scale cam 0 0 $canvas(zoom) $canvas(zoom)
	# 
	# Crea l'ovale a 0,0,0 e lo muove sulle coordinate, scalate, di cam(tmpPx)
	# 
	set camP1 [expr $cam(tmpP$v1)*$canvas(zoom)]
	set camP2 [expr [set $ncamP]*$canvas(zoom)]
	_drawCamPos $camP1 $camP2
	# 
	# Centra tutto cam nel 'grid center', necessario dopo changeView
	# 
	set cntCoords [$scCanv coords center]
	set cnt1 [lindex $cntCoords 0]
	set cnt2 [lindex $cntCoords 1]
	$scCanv move cam $cnt1 $cnt2
	$scCanv raise camLine all
	$scCanv raise camPos all
}

proc _drawCamPos {x y} {
	# 
	# Ricrea camPos, l'ovale di camera position, e lo posiziona in x y canvas
	# 
	global canvas scCanv
	$scCanv delete camPos cplabel
	$scCanv create text 0 0 -text "Camera position" -fill yellow -anchor sw \
		-tags "All cam cplabel"
	set cplX [expr $x-$canvas(camPosRadius)]
	set cplY [expr $y-$canvas(camPosRadius)]
	$scCanv move cplabel $cplX $cplY
	set ox1 -$canvas(camPosRadius); set oy1 -$canvas(camPosRadius)
	set ox2  $canvas(camPosRadius); set oy2  $canvas(camPosRadius)
	$scCanv create oval $ox1 $oy1 $ox2 $oy2 -outline yellow -fill yellow \
		-tags "cam camPos"
	$scCanv move camPos $x $y
}

proc _canvSnapViewTo {target} {
	global scCanv
	if {$target=="cameraPosition"} {
		set camLineCoords [$scCanv coords camLine]
		set p1x [format %.0f [lindex $camLineCoords 0]]; # intero!
		set p1y [format %.0f [lindex $camLineCoords 1]]; # intero!
		_canvSnapToXY $p1x $p1y
	}
	if {$target=="cameraLookAt"} {
		set camLineCoords [$scCanv coords camLine]
		set p1x [format %.0f [lindex $camLineCoords 2]]; # intero!
		set p1y [format %.0f [lindex $camLineCoords 3]]; # intero!
		_canvSnapToXY $p1x $p1y
	}
}

proc _canvSnapToXY {p1x p1y} {
	# 
	# p1x p1y sono le coordinate canvas del target
	# 
	global scCanv canvas
	# Centro window in canvasx/y
	set p2x [format %.0f [$scCanv canvasx [expr $canvas(canvasX)/2]]]
	set p2y [format %.0f [$scCanv canvasy [expr $canvas(canvasY)/2]]]
	
	# Calcolo delta x ---------------------------------------------
	if {$p1x>=0&&$p2x>=0} {; # p1 pos p2 pos
		if {$p1x>=$p2x} {set dx  [expr $p1x-$p2x]}
		if {$p1x< $p2x} {set dx -[expr $p2x-$p1x]}
	}
	if {$p1x>=0&&$p2x<=0} {; # p1 pos p2 neg
		set dx [expr abs($p1x)+abs($p2x)]
	}
	if {$p1x<=0&&$p2x<=0} {; # p1 neg p2 neg
		if {$p1x>=$p2x} {set dx -[expr $p2x-$p1x]}
		if {$p1x< $p2x} {set dx  [expr $p1x-$p2x]}
	}
	if {$p1x<=0&&$p2x>=0} {; # p1 neg p2 pos
		set dx -[expr abs($p1x)+abs($p2x)]
	}
	# Calcolo delta y ---------------------------------------------
	if {$p1y>=0&&$p2y>=0} {; # p1 pos p2 pos
		if {$p1y>=$p2y} {set dy  [expr $p1y-$p2y]}
		if {$p1y< $p2y} {set dy -[expr $p2y-$p1y]}
	}
	if {$p1y>=0&&$p2y<=0} {; # p1 pos p2 neg
		set dy [expr abs($p1y)+abs($p2y)]
	}
	if {$p1y<=0&&$p2y<=0} {; # p1 neg p2 neg
		if {$p1y>=$p2y} {set dy -[expr $p2y-$p1y]}
		if {$p1y< $p2y} {set dy  [expr $p1y-$p2y]}
	}
	if {$p1y<=0&&$p2y>=0} {; # p1 neg p2 pos
		set dy -[expr abs($p1y)+abs($p2y)]
	}
	$scCanv xview scroll $dx units
	$scCanv yview scroll $dy units
}

proc _canvSnapToCamLkt {} {
}

proc _canvZoom {what val} {
	global scCanv canvas
	# 
	# Centro della window contenente il canvas
	# 
	set xCenter [$scCanv canvasx [expr $canvas(canvasX)/2]]
	set yCenter [$scCanv canvasy [expr $canvas(canvasY)/2]]
	$scCanv scale $what $xCenter $yCenter $val $val
	set xPosLine [lindex [$scCanv coords camLine] 0]
	set yPosLine [lindex [$scCanv coords camLine] 1]
	_drawCamPos $xPosLine $yPosLine
	$scCanv config -scrollregion [$scCanv bbox all]
	set canvas(zoom) [expr $canvas(zoom)*$val]
}

proc _canvMark {x y} {
	global scCanv canv canvas
	if {$x<0||$x>$canvas(canvasX)} return
	if {$y<0||$y>$canvas(canvasY)} return
	set canv(currItem) [$scCanv find withtag current]
	set canv(currItemX) $x
	set canv(currItemY) $y
}

proc _canvDrag {x y camera} {
	# 
	# camera==1 muovo Position
	# camera==2 muovo LookAt
	# 
	global scCanv canv canvas
	if {$x<0||$x>$canvas(canvasX)} return
	if {$y<0||$y>$canvas(canvasY)} return
	set dx [expr $x - $canv(currItemX)]
	set dy [expr $y - $canv(currItemY)]
	if {$camera!=2} {$scCanv move $canv(currItem) $dx $dy}
	if {$camera==0} return
	if {$camera==1} _canvMoveCamLinePos
	if {$camera==2} {_canvMoveCamLineLkt $dx $dy}
	set canv(currItemX) $x
	set canv(currItemY) $y
}

proc _canvMoveCamLinePos {} {
	global scCanv canvas cam
	set ocds [$scCanv coords camPos]; # current oval coords
	set ocx [lindex $ocds 0]; # oval current x1
	set ocy [lindex $ocds 1]; # oval current y1
	set ovalxCenter [expr $ocx+$canvas(camPosRadius)]
	set ovalyCenter [expr $ocy+$canvas(camPosRadius)]
	set currLineCoors [$scCanv coords camLine]
	set currLktX [lindex $currLineCoors 2]
	set currLktY [lindex $currLineCoors 3]
	$scCanv coords camLine $ovalxCenter $ovalyCenter $currLktX $currLktY
	$scCanv coords cplabel $ocx $ocy
	_applyCameraCoords info1
}

proc _canvMoveCamLineLkt {dx dy} {
	global scCanv cam
	set currLineCoors [$scCanv coords camLine]
	set currPosX [lindex $currLineCoors 0]
	set currPosY [lindex $currLineCoors 1]
	set currLktX [lindex $currLineCoors 2]
	set currLktY [lindex $currLineCoors 3]
	set newLx [expr $currLktX + $dx]
	set newLy [expr $currLktY + $dy]
	$scCanv coords camLine $currPosX $currPosY $newLx $newLy
	$scCanv coords cllabel $newLx $newLy
	_applyCameraCoords info2
}

