proc _setInputFile {} {
	global inputFile
	set types {
    	{{Moray POV files}       {.pov}}
	}
	set fname [tk_getOpenFile -filetypes $types \
			-defaultextension pov -title "Select Moray-POV input file" -parent . \
			-initialdir $inputFile(dir) -initialfile $inputFile(name)
		]
	if {$fname==""} return
	set inputFile(dir) [file dirname $fname]
	set inputFile(name) [file tail $fname]
}

proc _setOutputFile {} {
	global outputFile
	set types {
    	{{MoVib files}       {.vib}}
	}
	set fname [tk_getSaveFile -filetypes $types \
			-defaultextension vib -title "Set VIB output file" -parent . \
			-initialdir $outputFile(dir) -initialfile $outputFile(name)
		]
	if {$fname==""} return
	set outputFile(dir) [file dirname $fname]
	set outputFile(name) [file tail $fname]
	set outputFile(log) [file rootname $outputFile(name)].log
	set outputFile(m2v) [file rootname $outputFile(name)].m2v
	set outputFile(geo) [file rootname $outputFile(name)].geo
}

proc _saveVLfilesAs {} {
	global outputFile done
	if {$done(parsing)==0} return
	set types {{{MoVib files}       {.vib}}}
	set fname [tk_getSaveFile -filetypes $types \
			-defaultextension vib -title "Save $outputFile(name) as" -parent . \
			-initialdir $outputFile(dir) -initialfile $outputFile(name)
		]
	if {$fname==""} return
	set outputFile(dir) [file dirname $fname]
	set outputFile(name) [file tail $fname]
	set outputFile(log) [file rootname $outputFile(name)].log
	set outputFile(m2v) [file rootname $outputFile(name)].m2v
	set outputFile(geo) [file rootname $outputFile(name)].geo
	_saveVLfiles
}

proc _saveVLfiles {} {
	global gui outputFile oText mess lbLights lbShapes lbDecs objList material \
		archiveFile tmpDir done
	if {$done(parsing)==0} return
	# 
	# vib file (contenuto di oText ed eventuale .geo)
	# 
	_cursor watch
	_saveGeoFile
	set f [open $outputFile(dir)/$outputFile(name) w]
	fconfigure $f -blocking 0
	puts -nonewline $f [$oText get 1.0 "end -1l"]
	close $f
	# 
	# log file (contenuto di mess)
	# 
	set f1 [open $outputFile(dir)/$outputFile(log) w]
	fconfigure $f1 -blocking 0
	set hlightRange [$mess tag nextrange hlight 1.0]
	$mess config -state normal
	$mess tag remove hlight 1.0 end; # altrimenti problemi con il loadVib
	$mess tag remove sel 1.0 end; # altrimenti problemi con il loadVib
	puts $f1 [$mess dump -tag 1.0 end]
	if {$hlightRange!=""} {
		$mess tag add hlight [lindex $hlightRange 0] [lindex $hlightRange 1]
	}
	$mess config -state disabled
	puts -nonewline $f1 [$mess get 1.0 end]
	close $f1
	# 
	# m2v file (gerarchia degli items per il tree)
	#
	# le prime 4 righe sono dedicate
	# le successive, fino alla riga == "... Materials", sono le vars di gerarchia
	# quelle successive a "... Materials" sono la definizione dell'array material()
	# 
	set f2 [open $outputFile(dir)/$outputFile(m2v) w]
	puts $f2 "Lights : [$lbLights get 0 end]"
	puts $f2 "Tops : [join [$lbShapes get 0 end]]"
	puts $f2 "Decs : [join [$lbDecs get 0 end]]"
	puts $f2 "Archives : $archiveFile(list)"
	foreach i $objList {global $i; puts $f2 "$i : [set $i]"}
	puts $f2 "--------------  Materials"
	for {set n 0} {$n<[array size material]} {incr n} {
		set ename [lindex [array names material] $n]
		puts $f2 "$ename [set material($ename)]"
	}
	puts $f2 ""; # a capo finale
	close $f2
	_cursor {}
	.f1.h config -text "Saved $outputFile(name)"
	wm title . "$gui(appTitle) - Editing $outputFile(name)"
	after 1000 {.f1.h config -text ""}
}

proc _saveGeoFile {} {
	# 
	# Sostituisce il link al file .geo con il nuovo nome di output
	# 
	global oText outputFile
	set tmpList {}
	_state $oText normal
	set geoi [$oText search "// Geometry file " 1.0]
	if {$geoi!=""} {
		set oldFnameLine [$oText get "$geoi +1l linestart" "$geoi +1l lineend"]
		# path completo:
		set oldFname [string trim [lindex $oldFnameLine end] \"]
		# deve essere il path completo:
		set newFname $outputFile(dir)/$outputFile(geo)
		_delLine "$geoi +1l"
		$oText insert "$geoi+1l" "ReadArchive \"$newFname\"\n"
		if {$oldFname!="$newFname"} {
			if [catch {file copy -force $oldFname $newFname} err] {
				tk_messageBox -message "$err"
			}
		}
	}
	_state $oText disabled
}

proc _openVLfiles {} {
	global outputFile
	set types {{{MoVib files}       {.vib}}}
	set fname [tk_getOpenFile -filetypes $types \
			-defaultextension vib -title "Load MoVib VIB file" -parent . \
			-initialdir $outputFile(dir) -initialfile $outputFile(name)
		]
	if {$fname==""} return
	# 
	# Controlli:
	# 
	if [catch {open $fname r} outputFile(openId)] {
		tk_messageBox -icon error -message "Cannot open $fname"
		return
	}
	fconfigure $outputFile(openId) -blocking 0
	catch {_parseVibFileInfo} infoFile
	if {$infoFile=="notMoVib"} {
		tk_messageBox -icon error -message "Not a MoVib main file"
		return
	}
	set m2vFile [file rootname $fname].m2v
	if ![file exists $m2vFile] {
		tk_messageBox -icon error -message "Cannot open [file rootname $fname].m2v.\n\
			Required by MoVib"
		return
	}
	# 
	# Finalmente possiamo continuare:
	# 
	set outputFile(dir) [file dirname $fname]
	set outputFile(name) [file tail $fname]
	set outputFile(log) [file rootname $outputFile(name)].log
	set outputFile(m2v) [file rootname $outputFile(name)].m2v
	set outputFile(geo) [file rootname $outputFile(name)].geo
	_loadVLfiles
}

proc _loadVLfiles {} {
	global gui outputFile oText mess lbLights lbShapes lbDecs objList lightList \
		material archiveFile done allObjList
	if [catch {open $outputFile(dir)/$outputFile(name) r} outputFile(openId)] {
		tk_messageBox -message \
			"Cannot open VIB file\n$outputFile(dir)/$outputFile(name)"
		return
	}
	_cursor watch
	_state $oText normal
	_state $mess normal
	_clearScene
	# 
	# Vib file:
	# 
	$oText insert end [read $outputFile(openId)]
	close $outputFile(openId)
	# 
	# Log file:
	# 
	set logFile $outputFile(dir)/$outputFile(log)
	if [file exists $logFile] {
		set lfile [open $logFile r]
		gets $lfile tagLine; # (la prima linea e' la definizione dei tags)
		$mess insert end [read -nonewline $lfile]
		close $lfile
		if {[lindex $tagLine 0]=="tagon"} {_setMessTags $tagLine}
	} else {
		$mess insert end "Log file unavailable.\n"
	}
	# 
	# Hierarchy file (.m2v):
	# 
	set hfile [open $outputFile(dir)/$outputFile(m2v) r]
	gets $hfile line1; # Luci
	set nlights [llength [lrange $line1 2 end]]
	.f1.lllb config -text "$nlights lights:"
	foreach i [lrange $line1 2 end] {
		$lbLights insert end $i
		lappend lightList [lindex $i end]
	}
	gets $hfile line2; # Toplevels
	set topList [lrange $line2 2 end]
	.f1.shlb config -text "[llength $topList] toplevel items:"
	foreach i [lrange $line2 2 end] {$lbShapes insert end $i}
	gets $hfile line3; # Declared
	set decList [lrange $line3 2 end]
	.f1.delb config -text "[llength $decList] declared items:"
	gets $hfile line4; # Archive files
	set archiveFile(list) [lrange $line4 2 end]
	foreach i [lrange $line3 2 end] {$lbDecs insert end $i}
	set objList [join [lappend topList $decList]]
	set iniMatLine "--------------  Materials"
	while {[gets $hfile line]>=0} {
		if {$line=="$iniMatLine"} {
			while {[gets $hfile line]>=0} {
				set oname [lindex $line 0]
				set omat [lindex $line 1]
				set material($oname) $omat
			}
		}
		set oname [lindex $line 0]
		global $oname
		set $oname [lrange $line 2 end]
	}
	close $hfile
	_buildTree 0
	_cursor {}
	_refresh
	_state $oText disabled
	_state $mess disabled
	wm title . "$gui(appTitle) - Editing $outputFile(name)"
	.f4.r.bt.b1 config -state normal; # Render e ViewLog buttons
	.f4.r.bt.b2 config -state normal
	set done(parsing) 1
	# _puts $allObjList
}

proc _setMessTags {tagsLine} {
	# 
	# si suppone che siano solo sequenze ordinate di tagon/tagoff ed il tag sia solo
	# un colore
	# 
	global mess
	set nOfActions [expr [llength $tagsLine]/3]
	for {set n 1} {$n<=$nOfActions} {incr n} {
		set action [lrange $tagsLine 0 2]
		if {[lindex $action 0]=="tagon"} {
			set iniI [lindex $action end]
			set tg [lindex $action 1]
		}
		if {[lindex $action 0]=="tagoff"} {
			set endI [lindex $action end]
			$mess tag add $tg $iniI $endI
			$mess tag configure $tg -foreground $tg
		}
		set tagsLine [lreplace $tagsLine 0 2]
	}
}

proc _parsePovFileInfo {} {
	global inputFile oText archiveFile
	for {set n 1} {$n<=3} {incr n} {gets $inputFile(openId) line}
	gets $inputFile(openId) line4
	gets $inputFile(openId) line
	gets $inputFile(openId) line
	gets $inputFile(openId) line7
	set ml1 "//  Moray V3.3a For Windows Copyright (c) 1993-2001 Lutz + Kretzschmar"
	set ml2 "//  Moray V3.5 For Windows Copyright (c) * Lutz + Kretzschmar"
	if {![string match $ml2 $line7]&&$line7!=$ml1} {
		tk_messageBox -message "Input file not in correct format.\nMoray3.3a or 3.5\
		POV exported file needed."
		return notMoray
	}
	set sceneInfo [lrange $line4 2 end]
	# 
	# ogni modifica nella seconda linea qui sotto va aggiornata in _parseVibFileInfo:
	# 
	$oText insert end "// VirtuaLight VIB file.\n"
	$oText insert end "// Generated by MoVib.\n"
	$oText insert end "// Converted from $inputFile(name)\n\n"
	$oText insert end "// Scene options \n\n"
	$oText insert end "// Archive files \n"
	foreach af $archiveFile(inilist) {$oText insert end "ReadArchive \"$af\"\n"}
	$oText insert end "\n"
	$oText insert end "// General options \n\n"
	$oText insert end "// Sky & Sun \n\n"
	_mess {} "Folder $inputFile(dir)\nInput file: $inputFile(name)\n"
	_mess {} "POV/Moray scene contains: $sceneInfo\n"
}

proc _parseVibFileInfo {} {
	# 
	# Prime linee del file per verificare che sia un output di MoVib,
	# $outputFile(openId) viene chiuso comunque:
	# 
	global outputFile
	gets $outputFile(openId) line
	gets $outputFile(openId) line
	close $outputFile(openId)
	if {$line!="// Generated by MoVib."} {return notMoVib}
}

proc _openGeoFile {} {
	global oText done tmpDir outputFile
	if {$done(geoFile)==0} {
		# 
		# e' la prima volta che viene creato.
		# ReadArchive deve essere il path completo!
		# 
		set outputFile(geoId) [open $tmpDir/MoVibTmpScene.geo w]
		set done(geoFile) 1
		_mess pink "Including geometry file (.geo).\n"
		$oText insert end "\n// Geometry file \n"
		$oText insert end "ReadArchive \"$tmpDir/MoVibTmpScene.geo\"\n"
	} else {
		# 
		# e' gia' stato creato, lo apriamo in append
		# 
		set outputFile(geoId) [open $tmpDir/MoVibTmpScene.geo a]
	}
	fconfigure $outputFile(geoId) -blocking 0
}

