proc _parseIncludeFile {line} {
	# 
	# Arriva la linea #include 'file.*'
	# Presuppongo che:
	# file.inc (quando file.pov e' l'input) contenga solo i materiali quindi lo salto;
	# gli altri siano meshes (union) di triangle || smooth_triangle
	# 
	global inputFile outputFile oText mess messLb stats gui tmpDir lbDecs done \
		userIncludes
	set fileName [lindex [split $line \"] 1]
	set fileNameTail [file tail $fileName]
	if {$fileNameTail=="[file rootname $inputFile(name)].inc"} return; # MATERIALS
	if $userIncludes {
		_mess orange "User included file not allowed ($fileName). Skipped\n"
		return
	}
	if {$fileNameTail=="$fileName"} {
		set ffname $inputFile(dir)/$fileName    ; # path relativo (solo nome file)
	} else {
		set ffname $fileName                    ; # path assoluto
	}
	if [catch {open $ffname r} f] {
		tk_messageBox -message "Include file not found:\n$inputFile(dir)/$fileName"
		_mess orange "Include file $fileName not found.\n"
		return
	}
	fconfigure $f -blocking 0
	gets $f line
	_openGeoFile
	# 
	# parsing principale:
	# 
	set addOper 0
	set tri 0
	while {[gets $f line]>=0} {
		if [string match {\#declare * =*\{} $line] {
			set name [lindex [lindex [split $line =] 0] end] 
			puts $outputFile(geoId) "Declare $name =\nSolid \["
			_addObjToList $lbDecs "$name"
			_mess grey85 "declared object ($name):\n\t"
			_refresh
			$oText insert end "\n// Definition of '$name' in geo file\n"
		}
		if [string match {*smooth_triangle \{} $line] {
			for {set n 1} {$n<=3} {incr n} {
				gets $f line
				set v$n [string trimleft [join [lindex [split $line >] 0]] <]
				set n$n [string trimleft [join [lindex [split $line >] 1]] ", <"]
			}
			if $addOper {puts $outputFile(geoId) +}
			puts -nonewline $outputFile(geoId) \
				"  Shape \[TriangularPatch (($v1),($n1),($v2),($n2),($v3),($n3))\]"
			set addOper 1
			incr tri
		}
		if [string match {* triangle \{*<*} $line] {
			set vn [lindex $line 1]
			set v1 [join [string trimleft [join [lindex [split $vn >] 0]] <]]
			set v2 [string trimleft [join [lindex [split $line >] 1]] ", <"]
			set v3 [string trimleft [join [lindex [split $line >] 2]] ", <"]
			if $addOper {puts $outputFile(geoId) +}
			puts -nonewline $outputFile(geoId) \
				"  Shape \[Polygon (3,($v1),($v2),($v3))\]"
			set addOper 1
			incr tri
		}
		if $gui(tcounter) {set messLb "TriPatches: $tri"; update}
		if {$line=="\}"} {
			puts $outputFile(geoId) "\n\]\n"
			_mess pink "$tri triangles, geometry included in .geo file.\n"
			set stats(tri) [expr $stats(tri) + $tri]
		}
	}
	close $f
	close $outputFile(geoId)
}

proc _readArchiveMngt {type w} {
	global archiveFile done
	if {$done(parsing)==0} return
	toplevel .ra
	wm geometry .ra +[winfo pointerx .]+[winfo pointery .]
	wm transient .ra $w
	wm title .ra "ReadArchive files"
	frame .ra.de
	listbox .ra.de.l -height 10 -width 30 -yscrollcommand {.ra.de.s set} \
		-selectmode single -exportselection false
	scrollbar .ra.de.s -command {.ra.de.l yview} -orient vert
	pack .ra.de -fill both -anchor s -expand 1
	pack .ra.de.l -side left -fill both -expand 1
	pack .ra.de.s -side right -fill y
	_insert2Butt .ra.bt 8 18 "Add ..." "Remove selected" _addArchFile _remArchFile
	_insertApplyDismiss .ra.bt2 .ra $w "_applyReadArchive [list $type] $w"
	grab .ra
	_listArchFiles $type
}

proc _listArchFiles {type} {
	global archiveFile oText
	foreach f $archiveFile(${type}list) {
		.ra.de.l insert end $f
	}
}

proc _remArchFile {} {
	# 
	# solo _applyReadArchive aggiorna archiveFile(list)
	# 
	set sele [.ra.de.l curselection]
	if {$sele==""} return
	.ra.de.l delete $sele
}

proc _addArchFile {} {
	# 
	# solo _applyReadArchive aggiorna archiveFile(list)
	# 
	global archiveFile
	set types {
    	{{VirtuaLight files} {.vs}}
    	{{VirtuaLight files} {.vib}}
		{{All files}         {*}}
	}
	set fname [tk_getOpenFile -filetypes $types \
			-defaultextension vs -title "Select VL archive file" -parent .ra \
			-initialdir $archiveFile(dir)
		]
	if {$fname==""} return
	set archiveFile(dir) [file dirname $fname]
	set name [file tail $fname]
	.ra.de.l insert end $name
	.ra.de.l see end
}

proc _applyReadArchive {type w} {
	global oText archiveFile
	# 
	# da Settings win:
	# 
	if {$type=="ini"} {
		set archiveFile(inilist) [.ra.de.l get 0 end]
		grab $w
		destroy .ra
		return
	}
	# 
	# da Scene -> ReadArchive:
	# 
	set i [$oText search "// Archive files " 1.0]
	if {$i==""} {
		tk_messageBox -message "Manca la linea di riferimento" -parent .ra
		return
	}
	set i [expr $i +1.0]
	set archiveFile(list) {}
	# 
	# cancella tutte le linee dopo "// Archive files " fino alla prima linea vuota:
	# 
	_state $oText normal
	set line [$oText get $i "$i lineend"]
	while {$line!=""} {
		$oText delete $i "$i +1l linestart"
		set line [$oText get $i "$i lineend"]
	}
	set il [.ra.de.l get 0 end]
	if {$il!=""} {
		foreach name $il {
			$oText insert $i "ReadArchive \"$name\"\n"
			set i [expr $i +1.0]
			lappend archiveFile(list) $name
		}
	}
	_state $oText disabled
	grab $w
	destroy .ra
}
