proc _createObjMatArray {} {
	# 
	# Crea un array 'material' con indici==nomeOggettiNelTree e valori==materiale
	# (nullo o $iniMat)
	# 
	global allObjList material objsWithIniMat iniMat
	array unset material
	foreach o $allObjList {
		set oname [file tail $o]
		set m {}
		if {[lsearch $objsWithIniMat $oname]!=-1} {set m $iniMat}
		set material($oname) $m
	}
}

proc _matsInScene {} {
	# 
	# restituisce la lista dei materiali attualmente in uso nella scena
	# si basa sull'array material()
	# richiamata da .matM.m postcommand
	# 
	global material
	set l {}
	.matM.m delete 0 end
	.tree.m1 delete 0 end
	foreach o [array names material] {
		set m $material($o)
		if {[lsearch $l $m]==-1 && $m!=""} {
			lappend l $m
			.matM.m add command -label $m -command "_queryObjsWithMat $m"
			.tree.m1 add command -label $m -command "_selectObjsWithMat $m"
		}
	}
}

proc _readCurrMaterial name {
	global material
	set m $material($name)
	return $m
}

proc _loadMaterialsFromArchive {filepath} {
	# 
	# matList e' la lista dei materiali presenti nella listbox, sottoforma di array
	# del tipo $matList(nome_mat)=="nome_del_file_vs_proprietario"
	# 
	global lbMats matList archiveFile
	set f [open $filepath r]
	while {[gets $f line]>=0} {
		set mtc1 [string match {Declare * = Shader *} $line]
		set mtc2 [string match {static * = Shader *} $line]
		set mtc3 [string match {Declare * =} $line]
		if {$mtc1||$mtc2} {
			set mname [lindex $line 1]
			if {[lsearch [array names matList] $mname]==-1} {
				$lbMats insert end $mname
				set matList($mname) $filepath
			}
		}
		if $mtc3 {
			gets $f line2
			if [string match {*Shader *} $line2] {
				set mname [lindex $line 1]
				if {[lsearch [array names matList] $mname]==-1} {
					$lbMats insert end $mname
					set matList($mname) $filepath
				}
			}
		}
	}
	close $f
}

proc _setMatToSele {op} {
	global lbMats TreeSelection archiveFile
	if {$TreeSelection==""} return
	set csele [$lbMats curselection]
	if {$csele==""&&$op=="add"} {tk_messageBox -message "Choose a material"; return}
	_cursor watch
	foreach o $TreeSelection {_setMatToObj $op multi $o}
	_cursor {}
}

proc _setMatToObj {op type pathName} {
	# 
	# pathName e' il nome completo dell'oggetto rispetto al tree
	# 
	global oText lbMats TreeSelection material
	set csele [$lbMats curselection]
	if {$csele==""&&$op=="add"} {tk_messageBox -message "Choose a material"; return}
	set name [file tail $pathName]
	set oldMat $material($name)
	if {$oldMat==""&&$op=="del"} return
	if {$op=="add"} {set mat [$lbMats get $csele]}
	set quadre 1
	# 
	# Toplevel:
	# 
	if {[llength [split $pathName /]]==2} {
		set srcTop [$oText search -regexp "// Toplevel .* $name " 1.0]
		set firstS [$oText search "\[" $srcTop]
		set lc 1; # counter delle linee
		set count 1; # counter delle parentesi quadre
		set topType [lindex [$oText get "$srcTop linestart" "$srcTop lineend"] 2]
		set qi [$oText get "$srcTop +2l linestart" "$srcTop +2l lineend"]
		if {$topType=="refObject"&&![string match {*\[*} $qi]} {
			# e' un refObject senza materiale ne' transf., quindi senza quadre.
			set i "$srcTop +2l linestart"
			set refObjNoMat 1
		} else {
			while {$count!=0} {; # ricerca l'ultima quadra chiusa
				set tline \
					[$oText get "$firstS + $lc l linestart" "$firstS + $lc l lineend"]
				set l1 [llength [split $tline \[]]
				if {$l1!=1} {set count [expr $count + ($l1 - 1)]}
				set l2 [llength [split $tline \]]]
				if {$l2!=1} {set count [expr $count - ($l2 - 1)]}
				set i [$oText index "$firstS + $lc l linestart"]
				incr lc
			}
			set refObjNoMat 0
		}
		# ora $i e' l'indice dove cancellare il materiale (se c'e') vecchio e inserire
		# quello nuovo
		set mline [$oText get "$i -1l linestart" "$i -1l lineend"]
		if {[llength $mline]==1&&$refObjNoMat==0} {; # ha gia' un materiale
			set quadre [_delMaterialLine "$i -1l" $name]
		}
		if {$op=="add"&&$oldMat!=""} {; # aveva gia' un materiale
			if {$quadre==1} {; # ci sono ancora le quadre
				_addMaterialLine $mat "$i -1l linestart" 0 {} {}
			} else {; # non ci sono piu' quadre
				_addMaterialLine $mat "$i -2l linestart" 1 {} {}
			}
			set material($name) $mat
		}
		if {$op=="add"&&$oldMat==""} {; # non aveva ancora un materiale
			set l1 [string trim [$oText get "$i linestart" "$i lineend"]]
			if {$l1=="\]"} {; # ci sono le quadre
				_addMaterialLine $mat $i 0 {} {}
			} else {; # non ci sono le quadre
				_addMaterialLine $mat $i 1 {} {}
			}
			set material($name) $mat
		}
		return
	}
	# 
	# Instance:
	# 
	set srcIns [$oText search "// Instance Object $name " 1.0]
	if {$srcIns!=""} {; # e' un Instance
		set sep [$oText get "$srcIns linestart" $srcIns]
		set optLine [string trim \
			[$oText get "$srcIns +2l linestart" "$srcIns +2l lineend"]]
		if {$optLine=="\["} {; # c'e' mat o transf.
			set optEnd [$oText search "\]" $srcIns]
			set operLine [$oText get "$optEnd linestart" "$optEnd lineend"]
			set oper [lindex $operLine end]
			set opLine [llength $operLine]
			set matLine [$oText get "$optEnd -1l linestart" "$optEnd -1l lineend"]
			if {[llength $matLine]==1} {; # c'e' gia' il materiale, cancelliamolo
				set oq [$oText index "$optEnd -1l linestart"]
				set quadre [_delMaterialLine $oq $name]
				if {$op=="del"&&$opLine==2&&$quadre==0} {; # c'era un operator
					_state $oText normal
					$oText insert "$srcIns +1l lineend" " $oper"
					_state $oText disabled
				}
			}
			if {$op=="add"&&$oldMat!=""} {; # aveva gia' un materiale
				if {$quadre==1} {; # ci sono ancora le quadre (transf.)
					set optEnd [$oText search "\]" $srcIns]
					_addMaterialLine $mat "$optEnd linestart" 0 {} $sep
				} else {; # non ci sono piu' quadre
					if {$opLine==2} {; # c'e' un operator dopo le quadre, e' in CSG
						_addMaterialLine $mat "$srcIns +2l linestart" 1 $oper $sep
					} else {
						_addMaterialLine $mat "$srcIns +2l linestart" 1 {} $sep
					}
				}
				set material($name) $mat
			}
			if {$op=="add"&&$oldMat==""} {; # senza materiale ma con transf.
				_addMaterialLine $mat "$optEnd linestart" 0 {} $sep
				set material($name) $mat
			}
		}
		if {$optLine!="\["&&$op=="add"} {; # non c'e' ne' mat ne' transf.
			set nameLine [$oText get "$srcIns +1l linestart" "$srcIns +1l lineend"]
			set oname [lindex $nameLine 0]
			set oper [lindex $nameLine end]
			set opLine [llength $nameLine]
			if {$opLine==2} {;# c'e' un operator, e' in CSG
				_state $oText normal
				$oText delete "$srcIns +1l linestart" "$srcIns +1l lineend"
				$oText insert "$srcIns +1l linestart" "  $oname"
				_state $oText disabled
				_addMaterialLine $mat "$srcIns +2l linestart" 1 $oper $sep
			} else {
				_addMaterialLine $mat "$srcIns +2l linestart" 1 {} $sep
			}
			set material($name) $mat
		}
		return
	}
	# 
	# Primitive:
	# 
	set srcPri [$oText search -regexp "// Primitive .* $name " 1.0]
	if {$srcPri!=""} {
		set sep [$oText get "$srcPri linestart" $srcPri]
		set ei [$oText search "\]" $srcPri]
		set eline [$oText get "$ei linestart" "$ei lineend"]
		set mline [$oText get "$ei -1l linestart" "$ei -1l lineend"]
		if {[llength $mline]==1} {; # c'e' il materiale
			_delMaterialLine "$ei -1l" $name
			if {$op=="add"} {_addMaterialLine $mat "$ei -1l linestart" 0 {} $sep}
		} else {
			if {$op=="add"} {_addMaterialLine $mat "$ei linestart" 0 {} $sep}
		}
		if {$op=="add"} {set material($name) $mat}
	}
}

proc _addMaterialLine {mat i withSquareBrkt oper sep} {
	# 
	# withSquareBrkt==1 aggiunge anche le quadre
	# oper!="" aggiunge l'operator di CSG dopo le quadre
	# 
	global oText
	_state $oText normal
	if {$withSquareBrkt==0} {$oText insert $i "$sep  $mat\n"}
		if {$withSquareBrkt==1} {
		if {$oper!=""} {	
			$oText insert $i "$sep\[\n$sep  $mat\n$sep\] $oper\n"
		} else {	
			$oText insert $i "$sep\[\n$sep  $mat\n$sep\]\n"
		}
	}
	_state $oText disabled
}

proc _delMaterialLine {i name} {
	# 
	# Cancella anche le quadre se la linea mat e' l'unica all'interno di esse
	# 
	global oText material
	if {$material($name)==""} return
	set quadre 1
	_state $oText normal
	$oText delete "$i linestart" "$i +1l linestart"
	set l1 [string trim [$oText get "$i linestart" "$i lineend"]]
	set l2 [string trim [$oText get "$i -1l linestart" "$i -1l lineend"]]
	if {$l1=="\]" && $l2=="\["} {; # nessun operator dopo la quadra chiusa
		$oText delete "$i -1l linestart" "$i linestart +1l"
		set quadre 0
	} elseif {[string match {\] ?} $l1]==1&&$l2=="\["} {; # c'e' un operator
		$oText delete "$i -1l linestart" "$i linestart +1l"
		set quadre 0
	}
	set material($name) {}
	_state $oText disabled
	return $quadre
}

proc _queryObjsMaterialsWin {} {
	global gui material done
	if {$done(parsing)==0} return
	_cursor watch
	toplevel .qm
 	wm geometry .qm +[winfo pointerx .]+[winfo pointery .]
	wm transient .qm .
	bind .qm <Escape> "grab .; destroy .qm"
	listbox .qm.l1 -height 18 -yscrollcommand [list .qm.sb set]
	listbox .qm.l2 -height 18 -yscrollcommand [list .qm.sb set]
	foreach l {.qm.l1 .qm.l2} {
		bind $l <B2-Motion> [list _bindDragTo %x %y .qm.l1 .qm.l2]
		bind $l <Button-2> [list _bindMark %x %y .qm.l1 .qm.l2]
	}
	scrollbar .qm.sb -orient vert -command [list _bindYview [list .qm.l1 .qm.l2]]
	pack .qm.l1 .qm.l2 -side left -fill both -expand 1
	pack .qm.sb -side left -fill y
	foreach i [array names material] {
		.qm.l1 insert end "$i"
		set m "$material($i)"
		if {$m==""} {set m "\[no material\]"}
		.qm.l2 insert end $m
		if {$m=="\[no material\]"} {
			.qm.l2 itemconfigure [expr [.qm.l2 index end] -1] -foreground red
		}
	}
	_cursor {}
	wm title .qm "Materials ([.qm.l1 size] objects)"
	focus .qm
	grab .qm
}

proc _bindDragTo {x y args} {
	foreach w $args {$w scan dragto $x $y}
}

proc _bindMark {x y args} {
	foreach w $args {$w scan mark $x $y}
}

proc _bindYview {lists args} {
	foreach l $lists {eval {$l yview} $args}
}

proc _queryObjsWithMat m {
	global material
	toplevel .qm
 	wm geometry .qm +[winfo pointerx .]+[winfo pointery .]
	wm transient .qm .
	bind .qm <Escape> "grab .; destroy .qm"
	listbox .qm.l1 -height 18 -yscrollcommand {.qm.sb set}
	scrollbar .qm.sb -orient vert -command {.qm.l1 yview}
	pack .qm.l1 -side left -fill both -expand 1
	pack .qm.sb -side left -fill y
	foreach i [array names material] {
		set om "$material($i)"
		if {$om=="$m"} {.qm.l1 insert end "$i"}
	}
	wm title .qm "Objects with '$m'"
	focus .qm
	grab .qm
}

proc _loadMatArchFile {} {
	global archiveFile
	set types {
    	{{Virtualight files}       {.vs}}
	}
	set fname [tk_getOpenFile -filetypes $types \
			-defaultextension vs -title "Select archive file" -parent . \
			-initialdir $archiveFile(dir)
		]
	if {$fname==""} return
	set archiveFile(dir) [file dirname $fname]
	_loadMaterialsFromArchive $fname
}

proc _viewMaterial {} {
	global lbMats matList commText
	set sele [$lbMats curselection]
	if {$sele==""} return
	set m [$lbMats get $sele]
	set file $matList($m)
	_messageText . "Material: $m ([file tail $file])" {} {}
	_messageTextInsert "Material '$m' from file:\n[file tail $file]\n"
	_messageTextInsert "###########################################\n\n"
	set f [open $file r]
	set count 0; # counter delle parentesi quadre
	while {[gets $f line]>=0} {
		set mtc1 [string match "Declare $m = Shader *" $line]
		set mtc2 [string match "static $m = Shader *" $line]
		if {$mtc1||$mtc2} {
			set l1 [llength [split $line \[]]
			if {$l1!=1} {set count [expr $count + ($l1 - 1)]}
			set l2 [llength [split $line \]]]
			if {$l2!=1} {set count [expr $count - ($l2 - 1)]}
			_messageTextInsert $line\n
			while {[gets $f line]>=0} {
				set l1 [llength [split $line \[]]
				if {$l1!=1} {set count [expr $count + ($l1 - 1)]}
				set l2 [llength [split $line \]]]
				if {$l2!=1} {set count [expr $count - ($l2 - 1)]}
				_messageTextInsert $line\n
				if {$count==0} {close $f; return}
			}
		}
	}
	close $f
}

proc _selectObjsWithMat m {
	global allObjList TreeSelection material
	_selectAll 0
	foreach i $allObjList {
		set oname [file tail $i]
		set omat $material($oname)
		if {$omat=="$m"} {
			Tree:drawselection $i
			lappend TreeSelection $i
		}
	}
	_selectedCount
}

proc _selectPatternWin {} {
	if [winfo exists .spt] return
	global oText
	toplevel .spt
	wm geometry .spt +[winfo pointerx .]+[winfo pointery .]
	wm title .spt "Select objects by pattern"
	wm transient .spt .
	bind .spt <Escape> "destroy .spt"
	entry .spt.en -width 30 -justify c -textvariable selPattern(val)
	checkbutton .spt.cb1 -text "Add to current selection" -var selPattern(add)
	checkbutton .spt.cb2 -text "Ignore case" -var selPattern(case)
	pack .spt.en -fill x -expand 1
	pack .spt.cb2 .spt.cb1 -anchor w
	_insertApplyDismiss .spt.ad .spt . "_selectPattern"
	focus .spt.en
}

proc _selectPattern {} {
	global allObjList selPattern TreeSelection
	if {$selPattern(val)==""} return
	if {$selPattern(add)==0} {_selectAll 0}
	if $selPattern(case) {
		set comm "string match -nocase"
	} else {
		set comm "string match "
	}
	foreach i $allObjList {
		set oname [file tail $i]
		catch {eval $comm $selPattern(val) $oname} mtc
		if {$mtc!=0&&$mtc!=1} {
			tk_messageBox -message "Invalid pattern" -icon error -parent .spt
			return
		}
		if $mtc {
			Tree:drawselection $i
			lappend TreeSelection $i
		}
	}
	_selectedCount
}
