proc _aboutM2vl {} {
	tk_messageBox -message "
	MoVib version 0.1\n\n\
	MoVib is a POVray scene parser and VIB editor.\n\
	It converts POVray scenes, exported by Moray modeler,\n\
	in VirtuaLight Interface (1.2) VIB scene ASCII format and it can be used as a\n\
	simple VIB scene editor.\n\n\
	MoVib is written in pure TclTk by Gigi Giorgi\n\
	Comments/suggestions are welcome! E-mail at\n    gigi.giorgi@ciaoweb.it
	"
}

proc _helpM2vl {} {
	_messageText . "MoVib help" {} {}
	_messageTextInsert "\
	Set the input Moray/POV file with the first 'yellow folder' button (or with\n\
	'File>Set POV input file').\n\
	Run POV-VIB conversion with the little white button (Parse input file).\n\
	Note that current VIB will be overwritten.\n\
	Set the VIB output file name with the second 'yellow folder' button\n\
	and save it with the red one.\n\n\
	Note that trying to parse manually edited POV files will probably cause\n\
	application errors.\n\
	MoVib is only able to (hopefully) correctly parse unmodified POV scenes\n\
	exported by Moray (see 'Help>Supported Moray objects' for details) and edit its\n\
	own VIB ASCII format.\n\n\
	You can drag listbox/text area and tree content while pressing and dragging\n\
	middle mouse button, like any other TclTk application (View menu allows to\n\
	show/hide sections and scrollbars of the application GUI).\n\
	Pressing right mouse button in any entry field automatically selects the whole\n\
	entry content.\n\n\
	While the mouse cursor is inside the 'Lights' listbox press the 3rd mouse\n\
	button to access the 'light properties' popup options. Options available for\n\
	lights will appear, which number depends on the current selection.\n\
	You can set multiple light properties by selecting one or more lights of the\n\
	same type and clicking the right mouse button. The light nearest the mouse\n\
	cursor will be the reference light for options values.\n\n\
	Selecting an item in the 'Toplevel' and 'Declared' listbox items will highlight\n\
	the corresponding element in the 'Messages', 'VIB' and 'Hiearchy tree' windows.\n\n\
	It is possible to display or hide the VIB, message, and render windows using\n\
	of F2, F3 and F4 keys (see 'View' menu in the toolbar) togheter with \n\
	scrollbars (F1).\n\n\
	In the VIB window (F2 toggle) you can find strings and go to specified line\n\
	numbers by pressing the right mouse button.\n\n\
	The hierarchy tree window allows the user to single/multi select items for\n\
	material assignement operations (left mouse button).\n\
	Selecting a 'father' item (colored in cyan) while holding down the Shify key\n\
	causes all its 'children' items to be selected/deselected.\n\
	Right mouse button opens a contextual popup menu for selection and material\n\
	operations. Control-RMB opens a selectable list containing all the objects\n\
	in tree.\n\
	If you want to expand the tree horizontally, hide Vib and\n\
	message frames by toggle them with F2 and F3.\n\n\
	The 'Materials' listbox lists all currently available material (press right\n\
	mouse button for options).\n\n\
	Rendering window is for VirtuaLight rendering tests. Press 'Render' button to\n\
	run vlight.exe. A little snapshot will be displayed in the 'Rendering' frame if\n\
	the option 'Make thumbnail' is active. Clicking on this thumbnail will bring\n\
	you to the rendered image.\n\n\
	Several VIB settings and utilities are available in the 'Scene' menu.\n\n\
	Please send comments/suggestions/bugs at gigi.giorgi@ciaoweb.it
	"
}

proc _exit {} {
	set answer [tk_messageBox -message "All unsaved data will be lost!\n\
		Do you really want to exit?" -parent . \
		-title "Exit MoVib" -type okcancel -icon question]
	switch -- $answer {
		ok exit
		cancel {}
	}
}

proc n {} {
	# Procedura nulla, ma serve !
}

proc _supported {} {
	_messageText . "MoVib help" {} {}
	_messageTextInsert "\
		Currently supported Moray objects:\n\
			\tCamera.\n\
			\tLights (point, area, spot).\n\
			\tCylinder, Cone, Superellipsoid, Disc, Torus, Sphere, Box,\n\
			\tLathe (linear_spline only), Prism (linear_spline & linear_sweep only),\n\
			\tPlane.\n\
			\tCSG (union, intersection, difference, merge).\n\
			\tMesh, Mesh2, Bicubic_patch.\n\
			\tRounded Disc, Rounded Cube (plug-in objects).\n\
		Unsupported objects:\n\
			\tPrism (quadratic, cubic), Text, Lathe (quadratic, cubic)\n\n\
		Note that any unsupported object can be converted in Mesh inside Moray.
		
	"
}

proc _topShell {} {
	# 
	# Window di standard output
	# (_puts ... usa questa shell in sostituzione di puts stdout)
	# 
	global shell
	toplevel .shell
	wm geometry .shell +0+700
	wm title .shell "Output shell (indirizzare con '_puts ...')"
	wm transient .shell .
	frame .shell.t
	set shell [text .shell.t.txt -xscrollcommand {.shell.sx set} -exportselect false \
		-yscrollcommand {.shell.t.sy set} -wrap none -width 76 -font "courier 10" \
		-height 20 -cursor left_ptr]
	scrollbar .shell.t.sy -command {$shell yview} -orient vert
	scrollbar .shell.sx -command {$shell xview} -orient horiz
	frame .shell.b
	button .shell.b.b1 -text "Clear" -command "
		$shell delete 1.0 end
		$shell insert end {> }
	"
	button .shell.b.b2 -text "Wrap" -command "$shell config -wrap word"
	button .shell.b.b3 -text "Unwrap" -command "$shell config -wrap none"
	pack $shell -side left -fill both -expand 1
	pack .shell.t -fill both -expand 1
	pack .shell.t.sy -fill y -expand 1
	pack .shell.sx .shell.b -fill x
	pack .shell.b.b1 .shell.b.b2 .shell.b.b3 -side right -anchor e
	$shell insert end "> "
}

proc _setImageDim {} {
	# 
	# Estrae da file.ini le dimensioni WxH dell'immagine per lo statement Camera:
	# 
	global inputFile oText
	set iniFileName [file rootname $inputFile(name)].ini
	if [catch {open $inputFile(dir)/$iniFileName r} fId] {
		tk_messageBox -message "Cannot open include file $iniFileName.\n\
			Needed for Image Output dimensions.\n\
			Width and Height set to 320 & 240"
		set wdt 320; set hgt 240
		$oText insert end "  Format ($wdt,$hgt)"
		return
	}
	fconfigure $fId -blocking 0
	while {[gets $fId line]>=0} {
		if {[lindex $line 0]=="Width"} {set wdt [lindex $line end]}
		if {[lindex $line 0]=="Height"} {set hgt [lindex $line end]}
	}
	$oText insert end "  Format ($wdt,$hgt)"
	close $fId
}

proc _vector {line} {
	# 
	# <n1, n2, n3> --> (n1, n2, n3)
	# <n1, n2> --> (n1, n2)
	# 
	set s1 [split [lindex [split $line "<"] end] ">"]
	set s2 [split [lindex $s1 0] ,]
	set s3 [join [join $s2 ,]]
	return "($s3)"
}

proc _vector2 {line} {
	# 
	# rotate||translate n1*x||y||z --> ($x, $y, $z)
	# es.: rotate 1.55*x
	# 
	set s [lindex $line end]
	set spl [split $s *]
	if {[lindex $spl 1]=="x"} {set s3 "[lindex $spl 0], 0, 0"}
	if {[lindex $spl 1]=="y"} {set s3 "0, [lindex $spl 0], 0"}
	if {[lindex $spl 1]=="z"} {set s3 "0, 0, [lindex $spl 0]"}
	return "($s3)"
}

proc _vector2a {line} {
	# 
	# rotate n*z rotate n*y rotate n*x --> ($x, $y, $z)
	# presumo che la sequenza in arrivo sia sempre rotate z, y, x
	# 
	set s1 [lindex [split [lindex $line 1] *] 0]
	set s2 [lindex [split [lindex $line 3] *] 0]
	set s3 [lindex [split [lindex $line 5] *] 0]
	return "($s3, $s2, $s1)"
}

proc _vector3 {line} {
	# 
	# Parentesi graffe
	# <n1, n2, n3> --> {n1, n2, n3}
	# <n1, n2> --> {n1, n2}
	# 
	set s1 [split [lindex [split $line "<"] end] ">"]
	set s2 [split [lindex $s1 0] ,]
	set s3 [join [join $s2 ,]]
	return "{$s3}"
}

proc _vectorColor {line} {
	# 
	# <n1, n2, n3> --> 'n1, n2, n3'
	# 
	set s1 [split [lindex [split $line "<"] end] ">"]
	set s2 [split [lindex $s1 0] ,]
	set s3 [join [join $s2 ,]]
	# 
	# <n1, n2, n3>*n4 --> 'n1, n2, n3'
	# 
	if [string match {*<*>*\**} $line] {
		set m [join [lindex [split $line *] end]]
		foreach v $s2 {lappend s [expr $v * $m]}
		set s3 [join $s ", "]
	}
	return "'$s3'"
}

proc _exaColor {r g b} {
	# 
	# RGB -> esadecimale
	# 
	set fred   [format %.0f [expr 65535*$r]]
	set fgreen [format %.0f [expr 65535*$g]]
	set fblue  [format %.0f [expr 65535*$b]]
	set color [format "#%04x%04x%04x" $fred $fgreen $fblue]
	return $color
}

proc _rgbColor {exacol} {
	# 
	# esadecimale -> RGB
	# 
	set rgbV1 [winfo rgb . $exacol]
	set r [format %.3f [expr [lindex $rgbV1 0]/65535.000]]
	set g [format %.3f [expr [lindex $rgbV1 1]/65535.000]]
	set b [format %.3f [expr [lindex $rgbV1 2]/65535.000]]
	return "$r $g $b"
}

proc _lightPosition {traList} {
	# traList deve essere nella forma:
	# "(5.94, -0.45, -2.56) (-3.8, 1.8, 0.6)" || "(5.94, -0.45, -2.56)"
	if {[llength $traList]==2} {
		set spl1 \
			[split [string trimleft [string trimright [lindex $traList 0] \)] \(] ,]
		set x1 [join [lindex $spl1 0]]
		set y1 [join [lindex $spl1 1]]
		set z1 [join [lindex $spl1 2]]
		set spl2 \
			[split [string trimleft [string trimright [lindex $traList 1] \)] \(] ,]
		set x2 [join [lindex $spl2 0]]
		set y2 [join [lindex $spl2 1]]
		set z2 [join [lindex $spl2 2]]
		set x [expr $x1 + $x2]
		set y [expr $y1 + $y2]
		set z [expr $z1 + $z2]
		return "($x, $y, $z)"
	} else {
		return [join $traList]
	}
}

proc _clearScene {} {
	global done mess oText lbShapes lbLights lbDecs lbMats trashNameList Tree objList \
		lightList currObj tree fathersList TreeSelection sky sun items skippedList \
		allObjList material objsWithIniMat stats messLb archiveFile iniMat tmpDir \
		vlrDir outputFile
	set trashNameList {}
	set done(camera) 0
	set done(parsing) 0
	set done(geoFile) 0
	$mess delete 1.0 end
	$oText delete 1.0 end
	$lbLights delete 0 end
	$lbShapes delete 0 end
	$lbDecs delete 0 end
	.f1.lllb config -text "Lights:"
	.f1.shlb config -text "Toplevel items:"
	.f1.delb config -text "Declared items:"
	set messLb {}
	foreach c $Tree($tree:/:children) {catch {Tree:delitem $tree /$c}}
	if [info exists objList] {
		foreach i $objList {
			if [info exists $i] {global $i; unset $i}
		}
	}
	set objList {}
	set allObjList {}
	set lightList {}
	set currObj {}
	set fathersList {}
	set items 0
	set TreeSelection {}
	set skippedList {}
	set sky(on) 0
	set sun(on) 0
	set objsWithIniMat {}
	set stats(tri) 0
	set stats(pri) 0
	set stats(skp) 0
	set stats(ins) 0
	set stats(bpt) 0
	array unset material
	set archiveFile(list) $archiveFile(inilist)
	_selectedCount
	.f4.r.bt.b1 config -state disabled; # Render e ViewLog buttons
	.f4.r.bt.b2 config -state disabled
	foreach f [glob -nocomplain $tmpDir/MoVib*] {file delete $f}
}

proc _parseTransformations {line} {
	# 
	# Cerca trasformazioni e crea uno stack di sottoforma di list,
	# in successione come le incontra nel pov e appendendo di volta in volta.
	# Per ogni nuovo Shape parsing occorre azzerare transfList in origine!
	# 
	global transfList
	if [string match {* scale *} $line] {
		catch {_transfStackScale $line} sca
		lappend transfList "Scale $sca"
	}
	if [string match {* rotate *} $line] {
		catch {_transfStackRotate $line} rot
		lappend transfList "Rotate $rot"
	}
	if [string match {* translate *} $line] {
		catch {_transfStackTranslate $line} tra
		lappend transfList "Translate $tra"
	}
}

proc _transfStackScale {line} {
	if {[llength $line]==2 && ![string match {*scale*<*>*} $line]} {; # casi scale n
		set sca "([lindex $line 1], [lindex $line 1], [lindex $line 1])"
	} elseif [string match {*scale*<*>*} $line] {; # casi scale <..>
		catch {_vector $line} sca
	}
	return $sca
}

proc _transfStackRotate {line} {
	set ll [llength $line]
	# 
	# casi rotate n*(axis) || (axis)*n:
	# 
	if {[string match {*rotate*\**} $line]&&$ll==2} {
		set l [split [lindex $line 1] *]
		set v1 [join [lindex $l 0]]; set v2 [join [lindex $l 1]]
		if [regexp {[xyz]} $v1 axis] {    ; # caso (axis)*n
			set op ""
			if [regexp {[-]} $v1] {set op -}; # caso -(axis)*n
			catch {_vector2 "rotate ${op}$v2*$axis"} rot
		}
		if [regexp {[xyz]} $v2 mtc2] {; # caso n*(axis)
			catch {_vector2 $line} rot
		}
	# 
	# casi 'inherit': rotate n*z rotate n*y rotate n*x:
	# 
	} elseif {[string match {*rotate*\**} $line]&&$ll==6} {
		catch {_vector2a $line} rot
	} elseif [string match {*rotate*<*>*} $line] {; # casi rotate <..>
		catch {_vector $line} rot
	}
	return $rot
}

proc _transfStackTranslate {line} {
	if [string match {*translate*\**} $line] {; # casi translate n*(axis)
		catch {_vector2 $line} tra
	} elseif [string match {*translate*<*>*} $line] {; # casi translate <..>
		catch {_vector $line} tra
	}
	return $tra
}

proc _writeMaterial {name} {
	global oText iniMat objsWithIniMat iniMatOn
	if !$iniMatOn return
	$oText insert end "  $iniMat\n"
	lappend objsWithIniMat $name
}

proc _puts {txt} {
	# 
	# Questa serve solo per la shell creata con _topShell
	# Bisogna cancellarla nella versione finale o fare attenzione che non ci sia nessun
	# chiamata a questa procedura, dato che richiama _topShell
	# 
	if ![winfo exists .shell] _topShell
	global shell
	set ii [$shell index "end -1l +2c"]
	$shell insert end $txt\n
	set ei [$shell index "end -2l lineend"]
	$shell insert end "> "
	$shell tag add txt $ii $ei
	$shell tag configure txt -background grey65
	$shell see end
}

proc _state {wgt state} {
	$wgt config -state $state
}

proc _refresh {} {
	global oText mess lbLights lbShapes lbDecs
	$oText see end
	$mess see end
	$lbLights see end
	$lbShapes see end
	$lbDecs see end
	update
}

proc _scrollbars {} {
	global gui
	if $gui(scrollbars) {
		pack .f2.t.t.sy -fill y -expand 1
		pack .f2.t.sx -fill x -side bottom
		pack .f2.t2.t.sy -fill y -expand 1
		pack .f2.t2.sx -fill x -side bottom
		pack .f3.y.sb -fill y -expand 1
		pack .f3.sb -fill x
		pack .f4.ll.s -side right -fill y
	} else {
		pack forget .f2.t.t.sy .f2.t.sx .f2.t2.t.sy .f2.t2.sx .f3.y.sb .f3.sb \
			.f4.ll.s
	}
}

proc _toggleFrame {var} {
	global gui
	if $gui($var) {set gui($var) 0} else {set gui($var) 1}
	_$var
}

proc _vibFrame {} {
	global gui
	if $gui(vibFrame) {
		pack .f2 -side left -fill both -expand 1 -padx 2 -pady 2
		pack forget .f3; # serve!
		pack .f3 -side right -fill both -padx 2 -pady 2
		pack .f2.t.t -fill both -expand 1
		pack .f2.t -fill both -expand 1 -side bottom
	} else {
		if {$gui(messFrame)==0} {
			pack forget .f2
			pack .f3 -side right -fill both -expand 1 -padx 2 -pady 2
		}
		pack forget .f2.t.t .f2.t
	}
}

proc _messFrame {} {
	global gui
	if $gui(messFrame) {
		pack .f2 -side left -fill both -expand 1 -padx 2 -pady 2
		pack forget .f3; # serve!
		pack .f3 -side right -fill both -padx 2 -pady 2
		pack .f2.t2.t -fill both -expand 1
		pack .f2.t2 -fill both -expand 1 -side top
	} else {
		if {$gui(vibFrame)==0} {
			pack forget .f2
			pack .f3 -side right -fill both -expand 1 -padx 2 -pady 2
		}
		pack forget .f2.t2.t .f2.t2
	}
}

proc _renderFrame {} {
	global gui
	if $gui(renderFrame) {
		pack .f4.r -side bottom -fill x
	} else {
		pack forget .f4.r
	}
	_packThumbLabel
}

proc _packThumbLabel {} {
	global render
	if $render(thumb) {
		pack .f4.r.im -fill both -expand 1 -after .f4.r.lb
	} else {
		pack forget .f4.r.im
	}
}

proc _mess {color text} {
	# 
	# Non aggiungere altro che non sia un tag colore! (vedi _setMessTags)
	# 
	global mess
	set currIndex [$mess index "end -1 chars"]
	$mess insert end $text
	if {$color=={}} return
	$mess tag add $color $currIndex "end -1 chars"
	$mess tag configure $color -foreground $color
}

proc _goToLine {} {
	if [winfo exists .goto] return
	global oText
	toplevel .goto
	wm geometry .goto +[winfo pointerx .]+[winfo pointery .]
	wm title .goto " Line"
	wm transient .goto .
	bind .goto <Escape> "destroy .goto"
	entry .goto.en -width 10 -justify c -textvar gtlVal
	bind .goto.en <Key-Return> _goToLineComm
	pack .goto.en
	focus .goto.en
}

proc _goToLineComm {} {
	global oText gtlVal
	$oText tag remove sel 1.0 end
	set tmpV [_isInt noNull $gtlVal {Line number} {}]
	if {$tmpV=="no"} {focus .goto.en; return} else {set gtlVal $tmpV}
	set i [$oText index $gtlVal.0]
	$oText see $i
	$oText mark set insert $i
	$oText tag add sel $i "$i +1l"
	destroy .goto
	focus $oText
}

proc _findInVib {} {
	if [winfo exists .fnd] return
	global oText
	toplevel .fnd
	wm geometry .fnd +[winfo pointerx .]+[winfo pointery .]
	wm title .fnd " Find text"
	wm transient .fnd .
	bind .fnd <Escape> "destroy .fnd"
	entry .fnd.en -width 30 -justify c
	pack .fnd.en -fill x -expand 1
	_insertApplyOtherDismiss .fnd.ad .fnd . Next "_applyFind 0 {}" "_applyFind 1 {}"
	focus .fnd.en
}

proc _applyFind {ini txt} {
	global oText
	if {$ini==0||$ini==1} {set txt [.fnd.en get]} else {set txt $txt}
	if {$txt==""} return
	set lgt [string length $txt]
	if {$ini==0} {set ii 1.0}
 	if {$ini==1||$ini==2} {set ii "[$oText index insert]"}
	set i [$oText search $txt $ii end]
	if {$i!=""} {
		_highlightTextInVib $i "$i +$lgt c"
	} else {
		if {$ini==0} {tk_messageBox -message "String not found" -parent .fnd}
		if {$ini==1} {tk_messageBox -message "Not found (end of file)" -parent .fnd}
		if {$ini==2} {tk_messageBox -message "Not found (end of file)" -parent .}
	}
}

proc _findDecInVib {} {
	global lbDecs
	set sele [$lbDecs curselection]
	if {$sele==""} return
	set name [$lbDecs get $sele]
	_applyFind 2 $name
}

proc _searchInText {type wgt y} {
	global mess oText tree
	set lbGet [$wgt get @0,$y]
	if {$lbGet==""} return
	if {$type!="light"} {
		set oname $lbGet
		_highlightTree $oname
	} else {
		set oname [lindex [split $lbGet] 1]; # e.g. 'PL L001' || 'PL L001 (declared)'
	}
	switch $type {
 		light {set str1 "Light \[(\]$oname\[)\]"; set str2 "// Light $oname "}
		toplv {
			set str1 "Toplevel .* \[(\]$oname\[)\]"
			set str2 "// Toplevel .* $oname "
		}
		decob {
			set str1 ".*declared .* \[(\]$oname\[)\]"
			set str2 "Declare $oname ="
		}
	}
 	set mii [$mess search -regexp $str1 1.0]
	set mei "$mii lineend"
	set vii [$oText search -regexp $str2 1.0]
	set vei "$vii lineend"
	_highlightTextinMess $mii $mei
	if {$vii==""} {
		# tk_messageBox -message "Error in searching VIB text"
		$oText tag remove sel 1.0 end
		.f1.h config -text "Object not found in VIB"
		after 1000 ".f1.h config -text {}"
		return
	}
	_highlightTextInVib $vii $vei
}
 
proc _highlightTextinMess {ii ei} {
	global mess
	$mess tag remove hlight 1.0 end
	if {$ii!=""} {
		$mess tag add hlight $ii $ei
		$mess tag configure hlight -underline 1 -foreground yellow
		$mess see $ei
	}
}

proc _highlightTextInVib {ii ei} {
	global oText
	focus $oText
	$oText tag remove sel 1.0 end
	$oText see $ii
	$oText mark set insert $ei
	$oText tag add sel $ii $ei	
}

proc _highlightTree {oname} {
	global tree
	$tree delete highl
	set bbox [$tree bbox $oname]
	if {$bbox==""} return
	set x1 [lindex $bbox 0]
	set y1 [lindex $bbox 1]
	set x2 [lindex $bbox 2]
	set y2 [lindex $bbox 3]
 	$tree create line 5 $y2 $x2 $y2 -fill orange -tags highl
	$tree lower highl
	set bbh [lindex [$tree bbox all] 3].0000000
	set yv [expr ($y1-20)/$bbh]
	$tree yview moveto $yv
}

proc _delLine {i} {
	# 
	# Cancella una linea completa in oText
	# 
	global oText
	$oText delete "$i linestart" "$i linestart +1l"
}

proc _isInt {type val name addOn} {
	# 
	# ritorna l'integer o 'no'.
	# Se riceve noNull come type, ritorna 'no' nel caso di stringa vuota
	# altrimenti ritorna stringa vuota
	# 
	catch {format %.0f $val} val; # es.: 059 error in [string is integer -strict $val]
	if {$type=="noNull"||$type=="fromFloat"} {
		set is [string is integer -strict $val]
	} else {
		set is [string is integer $val]
	}
	if $is {
		return [format %.0f $val]
	} else {
		if {$type!="fromFloat"} {_isErrorMessage $name integer $addOn}
		return no
	}
}

proc _isFlt {type val name addOn} {
	# 
	# verifica, mediante _isInt, che ci siano due integers separati da un punto
	# ritorna il float o 'no' da _isInt
	# 
	set val [string trim $val]; # nel caso di spazi vuoti da soli
	if {$val==""&&$type=="okNull"} return
	if {$val==""&&$type=="noNull"} {_isErrorMessage $name float $addOn; return no}
	set s [join [split $val .]]
	if {[llength $s]!=2} {_isErrorMessage $name float $addOn; return no}
	set i1 [string trim [lindex $s 0]]
	set i2 [string trim [lindex $s 1]]
	if {[_isInt fromFloat $i1 $name $addOn]=="no"} {
		_isErrorMessage $name float $addOn; return no
	}
	if {[_isInt fromFloat $i2 $name $addOn]=="no"} {
		_isErrorMessage $name float $addOn; return no
	}
	return "$i1.$i2"
}

proc _isErrorMessage {name type addOn} {
	tk_messageBox -message "'$name' value must be $type $addOn"
}

proc _cursor {type} {
	global oText mess
	. config -cursor $type
	$oText config -cursor $type
	$mess config -cursor $type
	update
}

proc _bindEnterLeave {w mess} {
	bind $w <Enter> ".f1.h config -text [list $mess]"
	bind $w <Leave> ".f1.h config -text {}"
}

proc _placeAtCenter {w} {
	wm withdraw $w
	update
	set x0 [winfo x .]
	set x1 [expr [winfo width .]/2]
	set x2 [expr [winfo width $w]/2]
	set x [expr $x0 + $x1 - $x2]
	set y0 [winfo y .]
	set y1 [expr [winfo height .]/2]
	set y2 [expr [winfo height $w]/2]
	set y [expr $y0 + $y1 - $y2]
	wm geometry $w +$x+$y
	wm deiconify $w
}

proc _placeAtCenterOfScreen {w} {
	wm withdraw $w
	update
	set screenW [winfo vrootwidth .]
	set screenH [winfo vrootheight .]
	set MoVibW [winfo width $w]
	set MoVibH [winfo height $w]
	set MoVibX [expr ($screenW/2)-($MoVibW/2)]
	set MoVibY [expr ($screenH/2)-($MoVibH/2)]
	wm geometry $w +$MoVibX+$MoVibY
	wm deiconify $w
}

proc _showVib {} {
	global oText done
	if {$done(parsing)==0} return
	_messageText . "Current VIB scene" {} {}
	_messageTextInsert [$oText get 1.0 "end -1l"]
}

proc _showGeo {} {
	global outputFile oText done
	if {$done(parsing)==0} return
	set gfi [$oText search "// Geometry file " 1.0]
	_messageText . "Current GEO mesh file" {} {}
	if {$gfi!=""} {
		set gf [split [lindex [$oText get "$gfi +1l" "$gfi +1l lineend"] end] \"]
		if [catch {open $gf r} f] {
			_messageTextInsert $f
			return
		} else {
			_messageTextInsert [read $f]
			close $f
		}
	} else {
		_messageTextInsert "No GEO file present.\n"
	}
}

proc _showPov {} {
	global oText inputFile
	if {$inputFile(name)==""} return
	set fn $inputFile(dir)/$inputFile(name)
	if ![file exists $fn] {
		tk_messageBox -message "POV file\n$fn\nnot found."
		return
	}
	set f [open $fn r]
	_messageText . "Current POV input file ($inputFile(name))" {} {}
	_messageTextInsert [read $f]
	close $f
}

proc _showLog {} {
	global mess
	_messageText . "Conversion log" {} {}
	_messageTextInsert [$mess get 1.0 end]
}

proc _addObjToList {lb name} {
	# 
	# aggiorna objList e crea una variabile per ogni toplevel/declared
	# objList e' la lista di tutti i toplevel/declared
	# Per ognuno di essi viene creata una variabile con lo stesso nome dell'oggetto
	# contentente i nomi dei 'figli'
	# 
	global objList currObj $name
	$lb insert end $name
	lappend objList $name
	set currObj $name
	set $name {}
	_refresh
}

proc _listSkipped {} {
	global skippedList
	_messageText . "[llength $skippedList] Unconverted items (ref. + instances)" {} {}
	if {$skippedList==""} {
		_messageTextInsert "None !\n"
		return
	}
	foreach sk $skippedList {_messageTextInsert "[join $sk]\n"}
}

